@echo off
REM WIM-based system cloning
REM This is useful when moving between disk sizes and/or BIOS types (BIOS <--> UEFI)
REM Author: Tyler Adkisson

set INSTALLER_SOURCE=%1
set CAPTURE_SOURCE=%2
set TARGET_PATH=%3
For /f "tokens=2-4 delims=/ " %%a in ('date /t') do (set CURRENT_DATE=%%c-%%a-%%b)

:checkargs
if [%INSTALLER_SOURCE%]==[] (
	echo Missing Windows Installation ISO path
	goto syntax
)
if [%CAPTURE_SOURCE%]==[] (
	echo Missing capture source path
	goto syntax
)
if [%TARGET_PATH%]==[] (
	echo Missing target path
	goto syntax
)

REM check of admin rights
net session >nul 2>&1
if %errorLevel% == 0 (
	echo Running as administrator
) else (
	echo This script must be run as Administrator
	goto cleanup
)

:createpaths
mkdir %TARGET_PATH% 2> nul

:copyinstaller
if exist %TARGET_PATH%\sources\boot.wim (
	echo Installer already exists in destination, skipping copy
	goto getname
)

echo Copying Windows install media files...
echo install.wim > tmp_exclude.txt
xcopy /e /h /r /k /b /i /exclude:tmp_exclude.txt %INSTALLER_SOURCE%\sources %TARGET_PATH%\sources
set ERROR=%errorlevel%
del tmp_exclude.txt
if not %ERROR%==0 goto error

REM del %TARGET_PATH%\sources\install.wim

:getname
echo.
if not exist %CAPTURE_SOURCE%\Windows\System32\config\SYSTEM (
	echo Warning: Cannot find source hostname. Continuing without
	set SOURCE_COMPUTERNAME="(unknown)"
	goto capture
)

echo Attempting to find source hostname
reg load HKLM\__tmp_system %CAPTURE_SOURCE%\Windows\System32\config\SYSTEM > nul
if not %errorlevel%==0 (
	echo Could not load source registery. Skipping
	goto capture
)
for /f "tokens=2*" %%i in ('Reg Query "HKLM\__tmp_system\ControlSet001\Control\ComputerName\ComputerName" /v "ComputerName" 2^>Nul') do set "SOURCE_COMPUTERNAME=%%j"
reg unload HKLM\__tmp_system > nul

echo Hostname: %SOURCE_COMPUTERNAME%

:capture
echo.
echo Capturing system image...
if exist %TARGET_PATH%\sources\install.wim (
	echo Warning: install.wim already exists; adding new image to it!
)

set WIM_NAME="System Image - %SOURCE_COMPUTERNAME% - %CURRENT_DATE%"
set WIM_DESCRIPTION="System image of %SOURCE_COMPUTERNAME% captured on %CURRENT_DATE%"

REM use wimlib-imagex if available
if exist wimlib-imagex.exe goto capturewimlib

:capturedism
REM Use custom config file, if exists
if exist config.ini (
	echo Using custom DISM exclusions
	set DISM_CONFIG=-/ConfigFile:config.ini
) else (
	set DISM_CONFIG=""
)

dism /Capture-Image /ImageFile:%TARGET_PATH%\sources\install.wim /CaptureDir:%CAPTURE_SOURCE% /Name:%WIM_NAME% /Description:%WIM_DESCRIPTION% %DISM_CONFIG%
if not %errorlevel%==0 goto error

goto postcapture

:capturewimlib
echo Wimlib available. Using it instead of DISM
if exist %TARGET_PATH%\sources\install.wim (
	set WIMLIB_CMD=append
) else (
	set WIMLIB_CMD=capture
)

REM Use custom config file, if exists
if exist config.ini (
	echo Using custom wimlib-imagex exclusions
	set WIMLIB_CONFIG=--config=config.ini
) else (
	set WIMLIB_CONFIG=""
)

wimlib-imagex.exe %WIMLIB_CMD% %CAPTURE_SOURCE% %TARGET_PATH%\sources\install.wim %WIM_NAME% %WIM_DESCRIPTION% --compress=fast --image-property DISPLAYNAME=%WIM_NAME% --image-property DISPLAYDESCRIPTION=%WIM_DESCRIPTION% %WIMLIB_CONFIG%
if not %errorlevel%==0 goto error

goto postcapture

:error
echo.
echo An error occurred. Stopping

goto cleanup

:postcapture

echo.
echo Resulting WIM file:
echo.
dir %TARGET_PATH%\sources\install.wim | findstr install.wim
echo.
echo Capture complete. Refer to the README.txt inside the destination directory for further instructions
echo.

goto writereadme

:writereadme
echo Welcome to the wonderful world of WIMs! > %TARGET_PATH%\README.txt
echo. >> %TARGET_PATH%\README.txt
echo To use, boot into a Windows Installation media as usual, then press Shift^+F10 >> %TARGET_PATH%\README.txt
echo to open a command prompt. >> %TARGET_PATH%\README.txt
echo. >> %TARGET_PATH%\README.txt
echo Run the setup.exe inside the sources directory next to this readme file to begin. >> %TARGET_PATH%\README.txt
echo Proceed with the setup as usual. If you've only captured a single image, there will >> %TARGET_PATH%\README.txt
echo be no edition selection screen. If you've captured multiple images, you can select >> %TARGET_PATH%\README.txt
echo one of them at the same point of the installer where you would usually select >> %TARGET_PATH%\README.txt
echo the Windows edition to install. >> %TARGET_PATH%\README.txt
echo. >> %TARGET_PATH%\README.txt
echo Note: Once the ^"installation^" is completed, you may need to adjust the time zone. >> %TARGET_PATH%\README.txt

goto cleanup

:syntax

echo.
echo WIM-clone script
echo.
echo Syntax: capture.bat ^<Windows install disc path^> ^<Capture source path^> ^<Target path^>
echo.
echo Example: capture.bat D:\ E:\ F:\destination
echo.
echo This means:
echo D:\ is a Windows installation disc
echo E:\ is the partition you wish to capture
echo F:\destination is the path to where the resulting installer and WIM are to be placed. Usually an external USB HDD


:cleanup
REM Unset our env vars
set INSTALLER_SOURCE=
set CAPTURE_SOURCE=
set TARGET=
set CURRENT_DATE=
set WIMLIB_CMD=
set WIMLIB_CONFIG=
set ERROR=
set SOURCE_COMPUTERNAME=
set DISM_CONFIG=
set SOURCE_COMPUTERNAME=
